@@ -26,11 +26,11 @@ Lavoco::Web::Editor - FastCGI app to edit flat-files.
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 $VERSION = eval $VERSION;
 
@@ -198,25 +198,25 @@ sub _init
         exit;
     }
 
-
-    if ( ! $config->{ files } )
-    {
-        _print_red( "[ FAIL ]\n" );
-        print "'files' attribute missing at top level.\nExiting...\n";
-        exit;
-    }
-
-    if ( ref $config->{ files } ne 'ARRAY' )
+    if ( exists $config->{ files } && ref $config->{ files } ne 'ARRAY' )
     {
         _print_red( "[ FAIL ]\n" );
         print "'files' attribute is not a list.\nExiting...\n";
         exit;
     }
 
-    if ( scalar @{ $config->{ files } } == 0 )
+    if ( exists $config->{ folders } && ref $config->{ folders } ne 'ARRAY' )
     {
-        _print_organge( "[ISSUE]\n" );
-        print "No 'files' defined in config, so no files to edit.\n";
+        _print_red( "[ FAIL ]\n" );
+        print "'folders' attribute is not a list.\nExiting...\n";
+        exit;
+    }
+
+    if ( exists $config->{ uploads } && ref $config->{ uploads } ne 'ARRAY' )
+    {
+        _print_red( "[ FAIL ]\n" );
+        print "'uploads' attribute is not a list.\nExiting...\n";
+        exit;
     }
 
     _print_green( "[  OK  ]\n" );
@@ -312,23 +312,28 @@ The config file is read for each and every request, so you can reasonably enable
 See the C<examples> directory for a sample JSON config file, similar to the following...
 
  {
+     "files"    : [
+         "app.json",
+         "site/style.css"
+     ],
      "folders"  : [
          "templates/content/organic",
          "templates/content/store"
      ],
-     "files"    : [
-         "app.json",
-         "site/style.css"
+     "uploads"  : [
+         "site/images"
      ],
      "password" : "foo",
      "salt"     : "abc123"
  }
 
-Two fields which drive the editor are C<folders> and C<files>, each of which is an array of paths, all relative to the base directory of the editor script.
+Three fields which drive the editor are C<files>, C<folders> and C<uploads>, each of which is an array of paths, all relative to the base directory of the editor script.
+
+Files in the C<files> list are editable, but the editor can not create new files in their respective containing directories.
 
 All visible files in the C<folders> are editable, but not sub-directories, you need to add those separately.  The editor can also create new files in each folder.
 
-Files in the C<files> list are editable, but the editor can not create new files in their respective containing directories.
+The editor can upload files into any of the C<uploads> folders.
 
 If there is a defined C<password> in the config, then this will be requested before a user can access the index page (listing all files that can be edited).
 
@@ -367,6 +372,8 @@ sub _handler
 
         $log->debug( "Requested path: " . $path ); 
 
+        $log->debug( $req->parameters );
+
         $stash{ app }->_reload_config( log => $log );
 
         ###############################
@@ -411,14 +418,35 @@ sub _handler
 
         if ( $template ne 'login.tt' )
         {
-            my @folders = ();
             my @files   = ();
+            my @folders = ();
+            my @uploads = ();
+
+            #########
+            # files #
+            #########
+
+            if ( exists $stash{ app }->config->{ files } )
+            {       
+                foreach my $file ( @{ $stash{ app }->config->{ files } } )
+                {
+                    $file =~ s/^\///g;   # remove leading slashes
+                    $file =~ s/\/$//g;   # remove trailing slashes
+
+                    $log->debug( "Processing file: " . $file );
+
+                    if ( -f $stash{ app }->_base . '/' . $file )
+                    {
+                        push @files, $file;
+                    }
+                }
+            }
 
             ###########
             # folders #
             ###########
 
-            if ( exists $stash{ app }->config->{ folders } && @{ $stash{ app }->config->{ folders } } )
+            if ( exists $stash{ app }->config->{ folders } )
             {
                 foreach my $folder ( @{ $stash{ app }->config->{ folders } } )
                 {
@@ -444,24 +472,32 @@ sub _handler
                 }
             }
 
-            #########
-            # files #
-            #########
+            ###########
+            # uploads #
+            ###########
 
-            if ( exists $stash{ app }->config->{ files } && @{ $stash{ app }->config->{ files } } )
-            {       
-                foreach my $file ( @{ $stash{ app }->config->{ files } } )
+            if ( exists $stash{ app }->config->{ uploads } )
+            {
+                foreach my $upload ( @{ $stash{ app }->config->{ uploads } } )
                 {
-                    $file =~ s/^\///g;   # remove leading slashes
-                    $file =~ s/\/$//g;   # remove trailing slashes
+                    $upload =~ s/^\///g;   # remove leading slashes
+                    $upload =~ s/\/$//g;   # remove trailing slashes
 
-                    $log->debug( "Processing file: " . $file );
+                    $log->debug( "Processing upload: " . $upload );
 
-                    my $path = $stash{ app }->_base . '/' . $file;
+                    my $path = $stash{ app }->_base . '/' . $upload;
 
-                    if ( -f $path )
+                    if ( -d $path )
                     {
-                        push @files, $file;
+                        my %upload = ( path => $upload, files => [ ] );
+
+                        opendir( my $dh, $path ) || $log->debug("Can't opendir $path: $!");
+
+                        push @{ $upload{ files } }, sort { $a cmp $b } grep { ! -d ( $stash{ app }->_base . '/' . $upload . '/' . $_ ) } grep { $_ !~ /^\./ } readdir( $dh );
+
+                        closedir( $dh );
+
+                        push @uploads, \%upload;
                     }
                 }
             }
@@ -470,7 +506,60 @@ sub _handler
             # if we've requested a file, edit it #
             ######################################
 
-            if ( exists $req->parameters->{ folder } )
+            if ( exists $req->parameters->{ file } && ! exists $req->parameters->{ folder } && ! $req->parameters->{ upload } )
+            {
+                foreach my $file ( @files )
+                {
+                    next if $file ne $req->parameters->{ file };
+
+                    $stash{ file } = $req->parameters->{ file };
+
+                    $template = 'edit.tt';
+
+                    if ( ! exists $req->parameters->{ content } )
+                    {
+                        $log->debug( "Reading content of " . $stash{ app }->_base .  '/' . $file );
+
+                        $stash{ content } = read_file( $stash{ app }->_base . '/' . $file, { binmode => ':utf8' } );
+                    }
+                    else
+                    {
+
+                        if ( $req->parameters->{ file } =~ /\.json/ )
+                        {
+                            $log->debug( "It's a JSON file" );
+
+                            #########################
+                            # basic json validation #
+                            #########################
+
+                            eval {
+                                my $json = JSON->new;
+
+                                $json->relaxed( 1 );
+
+                                $json->decode( $req->parameters->{ content } );
+                            };
+                            
+                            $log->debug( $@ ) if $@;
+
+                            $stash{ error } = $@ if $@;
+                        }
+
+                        if ( ! exists $stash{ error } )
+                        {
+                            write_file( $stash{ app }->_base . '/' . $file, { binmode => ':utf8' }, $req->parameters->{ content } );
+
+                            $stash{ success } = "Saved OK";
+                        }
+
+                        $stash{ file } = $req->parameters->{ file };
+
+                        $stash{ content } = $req->parameters->{ content };
+                    }                    
+                }
+            }
+            elsif ( exists $req->parameters->{ folder } )
             {
                 foreach my $folder ( @folders )
                 {
@@ -535,62 +624,41 @@ sub _handler
                     }
                 }
             }
-            elsif ( exists $req->parameters->{ file } )
+            elsif ( exists $req->parameters->{ upload } )
             {
-                foreach my $file ( @files )
+                foreach my $upload ( @uploads )
                 {
-                    next if $file ne $req->parameters->{ file };
+                    next if $upload->{ path } ne $req->parameters->{ upload };
 
-                    $stash{ file } = $req->parameters->{ file };
+                    $stash{ upload } = $req->parameters->{ upload };
 
-                    $template = 'edit.tt';
-
-                    if ( ! exists $req->parameters->{ content } )
+                    if ( my $uploaded = $req->upload('file') )
                     {
-                        $log->debug( "Reading content of " . $stash{ app }->_base .  '/' . $file );
+                        $log->debug( "Moving file from " . $uploaded->path . " to " . $stash{ app }->_base . '/' . $upload->{ path } . '/' . $uploaded->filename );
 
-                        $stash{ content } = read_file( $stash{ app }->_base . '/' . $file, { binmode => ':utf8' } );
+                        rename $uploaded->path, $stash{ app }->_base . '/' . $upload->{ path } . '/' . $uploaded->filename;
+
+                        $res->redirect( '/' );
+
+                        return $res->finalize;
                     }
-                    else
+    
+                    if ( exists $req->parameters->{ delete } )
                     {
+                        $log->debug( "Attempting to delete " . $stash{ app }->_base . '/' . $upload->{ path } . '/' . $req->parameters->{ delete } );
 
-                        if ( $req->parameters->{ file } =~ /\.json/ )
-                        {
-                            $log->debug( "It's a JSON file" );
+                        unlink $stash{ app }->_base . '/' . $upload->{ path } . '/' . $req->parameters->{ delete };
 
-                            #########################
-                            # basic json validation #
-                            #########################
+                        $res->redirect( '/' );
 
-                            eval {
-                                my $json = JSON->new;
-
-                                $json->relaxed( 1 );
-
-                                $json->decode( $req->parameters->{ content } );
-                            };
-                            
-                            $log->debug( $@ ) if $@;
-
-                            $stash{ error } = $@ if $@;
-                        }
-
-                        if ( ! exists $stash{ error } )
-                        {
-                            write_file( $stash{ app }->_base . '/' . $file, { binmode => ':utf8' }, $req->parameters->{ content } );
-
-                            $stash{ success } = "Saved OK";
-                        }
-
-                        $stash{ file } = $req->parameters->{ file };
-
-                        $stash{ content } = $req->parameters->{ content };
-                    }                    
+                        return $res->finalize;
+                    }
                 }
             }
 
-            $stash{ folders } = \@folders;
             $stash{ files   } = \@files;
+            $stash{ folders } = \@folders;
+            $stash{ uploads } = \@uploads;
         }
 
         ##############################
@@ -717,9 +785,29 @@ EOF
     [% END %]
                 </ul>
             </li>
-[% END %]            
+[% END %]
         </ul>
 
+[% IF uploads.size %]
+        <h3>Uploads</h3>
+        <ul>
+    [% FOREACH upload IN uploads %]
+            <li>
+                <strong>[% upload.path | html %]/</strong>
+                <form action="/" enctype="multipart/form-data" method="post">
+                    <input type="hidden" name="upload" value="[% upload.path | html %]">
+                    <input type="file" name="file">
+                    <input type="submit" value="Upload">
+                </form>
+                <ul>
+    [% FOREACH file IN upload.files %]
+                    <li>[% file %] [ <a href="/?upload=[% upload.path | html %]&amp;delete=[% file | html %]">delete</a> ]</li>
+    [% END %]
+                </ul>
+            </li>
+    [% END %]
+        </ul>
+[% END %]
     </body>
 
 </html>
